==> v0.0.1.tar.gz: maze-generator/src/main/java/ch/fritteli/labyrinth/generator/model/Labyrinth.java <==
package ch.fritteli.labyrinth.generator.model;

import io.vavr.control.Option;
import lombok.Getter;
import lombok.NonNull;

import java.util.Deque;
import java.util.LinkedList;
import java.util.Random;

public class Labyrinth {
    private final Tile[][] field;
    @Getter
    private final int width;
    @Getter
    private final int height;
    @Getter
    private final long randomSeed;
    private final Random random;
    @Getter
    private final Position start;
    @Getter
    private final Position end;

    public Labyrinth(final int width, final int height) {
        this(width, height, System.nanoTime());
    }

    public Labyrinth(final int width, final int height, final long randomSeed) {
        if (width <= 0 || height <= 0) {
            throw new IllegalArgumentException("widht and height must be >=1");
        }
        this.width = width;
        this.height = height;
        this.randomSeed = randomSeed;
        this.random = new Random(randomSeed);
        this.field = new Tile[width][height];
        this.start = new Position(0, 0);
        this.end = new Position(this.width - 1, this.height - 1);
        this.initField();
        this.generate();
    }

    @NonNull
    public Option<Tile> getTileAt(@NonNull final Position position) {
        return this.getTileAt(position.getX(), position.getY());
    }

    @NonNull
    public Option<Tile> getTileAt(final int x, final int y) {
        if (x < 0 || y < 0 || x >= this.width || y >= this.height) {
            return Option.none();
        }
        return Option.of(this.field[x][y]);
    }

    @NonNull
    Tile getStartTile() {
        return this.getTileAt(this.start).get();
    }

    @NonNull
    Tile getEndTile() {
        return this.getTileAt(this.end).get();
    }

    private void initField() {
        for (int x = 0; x < this.width; x++) {
            this.field[x] = new Tile[this.height];
            for (int y = 0; y < this.height; y++) {
                final Tile tile = new Tile();
                this.hardenWalls(tile, x, y);
                this.field[x][y] = tile;
            }
        }
    }

    private void hardenWalls(@NonNull final Tile tile, final int x, final int y) {
        if (x == 0) {
            tile.preventDiggingToOrFrom(Direction.LEFT);
        }
        if (x == this.width - 1) {
            tile.preventDiggingToOrFrom(Direction.RIGHT);
        }
        if (y == 0) {
            tile.preventDiggingToOrFrom(Direction.TOP);
        }
        if (y == this.height - 1) {
            tile.preventDiggingToOrFrom(Direction.BOTTOM);
        }
    }

    private void generate() {
        new Generator().run();
    }

    private class Generator {
        private final Deque<Position> positions = new LinkedList<>();

        void run() {
            this.preDig();
            this.dig();
            this.postDig();
        }

        private void preDig() {
            final Tile endTile = Labyrinth.this.getEndTile();
            endTile.enableDiggingToOrFrom(Direction.BOTTOM);
            endTile.digFrom(Direction.BOTTOM);
            this.positions.push(Labyrinth.this.end);
        }

        private void dig() {
            while (!this.positions.isEmpty()) {
                final Position currentPosition = this.positions.peek();
                final Tile currentTile = Labyrinth.this.getTileAt(currentPosition).get();
                final Option<Direction> directionToDigTo = currentTile.getRandomAvailableDirection(Labyrinth.this.random);
                if (directionToDigTo.isDefined()) {
                    final Direction digTo = directionToDigTo.get();
                    final Direction digFrom = digTo.invert();
                    final Position neighborPosition = currentPosition.move(digTo);
                    final Tile neighborTile = Labyrinth.this.getTileAt(neighborPosition).get();
                    if (currentTile.digTo(digTo) && neighborTile.digFrom(digFrom)) {
                        // all ok!
                        this.positions.push(neighborPosition);
                        if (neighborPosition.equals(Labyrinth.this.start)) {
                            this.markSolution();
                        }
                    } else {
                        // Hm, didn't work.
                        currentTile.undigTo(digTo);
                        currentTile.preventDiggingToOrFrom(digTo);
                    }
                } else {
                    this.positions.pop();
                }
            }
        }

        private void markSolution() {
            this.positions.forEach(position -> Labyrinth.this.getTileAt(position).get().setSolution());
        }

        private void postDig() {
            final Tile startTile = Labyrinth.this.getStartTile();
            startTile.enableDiggingToOrFrom(Direction.TOP);
            startTile.digTo(Direction.TOP);
        }
    }
}

==> v0.0.6.tar.gz: maze-generator/src/main/java/ch/fritteli/labyrinth/generator/model/Labyrinth.java <==
package ch.fritteli.labyrinth.generator.model;

import io.vavr.control.Option;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NonNull;
import lombok.ToString;

import java.util.Deque;
import java.util.LinkedList;
import java.util.Random;

@EqualsAndHashCode
@ToString
public class Labyrinth {
    private final Tile[][] field;
    @Getter
    private final int width;
    @Getter
    private final int height;
    @Getter
    private final long randomSeed;
    @EqualsAndHashCode.Exclude
    private final Random random;
    @Getter
    private final Position start;
    @Getter
    private final Position end;

    public Labyrinth(final int width, final int height) {
        this(width, height, System.nanoTime());
    }

    public Labyrinth(final int width, final int height, final long randomSeed) {
        if (width <= 1 || height <= 1) {
            throw new IllegalArgumentException("width and height must be >1");
        }
        this.width = width;
        this.height = height;
        this.randomSeed = randomSeed;
        this.random = new Random(randomSeed);
        this.field = new Tile[width][height];
        this.start = new Position(0, 0);
        this.end = new Position(this.width - 1, this.height - 1);
        this.initField();
        this.generate();
    }

    /**
     * INTERNAL API.
     * Exists only for deserialization. Not to be called from user code.
     */
    private Labyrinth(@NonNull final Tile[][] field, final int width, final int height, final long randomSeed) {
        this.field = field;
        this.width = width;
        this.height = height;
        this.randomSeed = randomSeed;
        this.random = new Random(randomSeed);
        this.start = new Position(0, 0);
        this.end = new Position(this.width - 1, this.height - 1);
    }

    @NonNull
    public Option<Tile> getTileAt(@NonNull final Position position) {
        return this.getTileAt(position.getX(), position.getY());
    }

    @NonNull
    public Option<Tile> getTileAt(final int x, final int y) {
        if (x < 0 || y < 0 || x >= this.width || y >= this.height) {
            return Option.none();
        }
        return Option.of(this.field[x][y]);
    }

    @NonNull
    Tile getStartTile() {
        return this.getTileAt(this.start).get();
    }

    @NonNull
    Tile getEndTile() {
        return this.getTileAt(this.end).get();
    }

    private void initField() {
        for (int x = 0; x < this.width; x++) {
            this.field[x] = new Tile[this.height];
            for (int y = 0; y < this.height; y++) {
                final Tile tile = new Tile();
                this.hardenWalls(tile, x, y);
                this.field[x][y] = tile;
            }
        }
    }

    private void hardenWalls(@NonNull final Tile tile, final int x, final int y) {
        if (x == 0) {
            tile.preventDiggingToOrFrom(Direction.LEFT);
        }
        if (x == this.width - 1) {
            tile.preventDiggingToOrFrom(Direction.RIGHT);
        }
        if (y == 0) {
            tile.preventDiggingToOrFrom(Direction.TOP);
        }
        if (y == this.height - 1) {
            tile.preventDiggingToOrFrom(Direction.BOTTOM);
        }
    }

    private void generate() {
        new Generator().run();
    }

    private class Generator {
        private final Deque<Position> positions = new LinkedList<>();

        void run() {
            this.preDig();
            this.dig();
            this.postDig();
        }

        private void preDig() {
            final Tile endTile = Labyrinth.this.getEndTile();
            endTile.enableDiggingToOrFrom(Direction.BOTTOM);
            endTile.digFrom(Direction.BOTTOM);
            this.positions.push(Labyrinth.this.end);
        }

        private void dig() {
            while (!this.positions.isEmpty()) {
                final Position currentPosition = this.positions.peek();
                final Tile currentTile = Labyrinth.this.getTileAt(currentPosition).get();
                final Option<Direction> directionToDigTo = currentTile.getRandomAvailableDirection(Labyrinth.this.random);
                if (directionToDigTo.isDefined()) {
                    final Direction digTo = directionToDigTo.get();
                    final Direction digFrom = digTo.invert();
                    final Position neighborPosition = currentPosition.move(digTo);
                    final Tile neighborTile = Labyrinth.this.getTileAt(neighborPosition).get();
                    if (currentTile.digTo(digTo) && neighborTile.digFrom(digFrom)) {
                        // all ok!
                        this.positions.push(neighborPosition);
                        if (neighborPosition.equals(Labyrinth.this.start)) {
                            this.markSolution();
                        }
                    } else {
                        // Hm, didn't work.
                        currentTile.undigTo(digTo);
                        currentTile.preventDiggingToOrFrom(digTo);
                    }
                } else {
                    this.positions.pop();
                }
            }
        }

        private void markSolution() {
            this.positions.forEach(position -> Labyrinth.this.getTileAt(position).get().setSolution());
        }

        private void postDig() {
            final Tile startTile = Labyrinth.this.getStartTile();
            startTile.enableDiggingToOrFrom(Direction.TOP);
            startTile.digTo(Direction.TOP);
        }
    }
}
